using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using Tao.FreeGlut;
using Tao.OpenGl;


namespace glut1
{
    class Program
    {
        // mousing around
        static bool MouseIsDown=false;
        static int  LastX;
        static int  LastY;
        static float v1 = 0.0f;
        static float v2 = 0.0f;
        // Golden ratio
        static float t = 1.61803398f;
        // holding a texture, need a reference variable
        private static int[] texture = new int[1];

#region data
        // 4 arrays: 
        // normal6: normals on the 20 polygons with 6 edges
        // normal5: normals on the 12 polygons with 5 edges
        // pol6: corners of the 20  polygons with 6 edges
        // pol5: corners of the 12  polygons with 5 edges
        // tex6: coordinates for texture alignment to polygons with 6 edges
        // tex5: coordinates for texture alignment to polygons with 5 edges
        
        // normals texture coordinates are calculated in PrepareData()
        static float[,] normal6 = new float[20, 3];
        static float[,] normal5 = new float[12, 3];
        static float[,] tex6 = new float[6, 2];
        static float[,] tex5 = new float[5, 2]; // not calculated and not used


        static float[, ,] pol6 = new float[20, 6, 3] { 
            {   //1
                {-2/3.0f,      -1/3.0f-2*t/3.0f,  -t/3.0f},
                {-1/3.0f,      -2/3.0f-t/3.0f,   -2*t/3.0f},
                {1/3.0f,       -2/3.0f-t/3.0f,   -2*t/3.0f}, 
                {2/3.0f,       -1/3.0f-2*t/3.0f,  -t/3.0f}, 
                {1/3.0f,       -t,         0.0f}, 
                {-1/3.0f,      -t,         0.0f}
            },
            {   //2
                {-2/3.0f-t/3.0f,  -2*t/3.0f,       -1/3.0f},
                {-1/3.0f-2*t/3.0f, -t/3.0f,       -2/3.0f},
                {-2*t/3.0f,     -1/3.0f,       -2/3.0f-t/3.0f},
                {-t/3.0f,      -2/3.0f,       -1/3.0f-2*t/3.0f},
                {-1/3.0f,      -2/3.0f-t/3.0f,   -2*t/3.0f},
                {-2/3.0f,      -1/3.0f-2*t/3.0f,  -t/3.0f}
            },
            {   //3
                {-1/3.0f,      -t,         0.0f},
                {1/3.0f,       -t,         0.0f},
                {2/3.0f,       -1/3.0f-2*t/3.0f,  t/3.0f},
                {1/3.0f,       -2/3.0f-t/3.0f,   2*t/3.0f},
                {-1/3.0f,      -2/3.0f-t/3.0f,   2*t/3.0f},
                {-2/3.0f,      -1/3.0f-2*t/3.0f,  t/3.0f}
            },
            {   //4
                {-2/3.0f,      -1/3.0f-2*t/3.0f,  t/3.0f},
                {-1/3.0f,      -2/3.0f-t/3.0f,   2*t/3.0f},
                {-t/3.0f,      -2/3.0f,       1/3.0f+2*t/3.0f},
                {-2*t/3.0f,     -1/3.0f,       2/3.0f+t/3.0f},
                {-1/3.0f-2*t/3.0f, -t/3.0f,       2/3.0f},
                {-2/3.0f-t/3.0f,  -2*t/3.0f,      1/3.0f}
            },
            {   //5
                {-2/3.0f-t/3.0f,    -2*t/3.0f,    1/3.0f},
                {-1/3.0f-2*t/3.0f,   -t/3.0f,     2/3.0f},
                 {-t,          0.0f,        1/3.0f},
                {-t,          0.0f,        -1/3.0f},
                {-1/3.0f-2*t/3.0f,   -t/3.0f,     -2/3.0f},
                {-2/3.0f-t/3.0f,   -2*t/3.0f,     -1/3.0f}
           },
            {   //6
                {-1/3.0f,        t,        0.0f},
                {1/3.0f,         t,        0.0f},
                {2/3.0f,         1/3.0f+2*t/3.0f, -t/3.0f},
                {1/3.0f,         2/3.0f+t/3.0f,  -2*t/3.0f},
                {-1/3.0f,        2/3.0f+t/3.0f,  -2*t/3.0f},
                {-2/3.0f,        1/3.0f+2*t/3.0f, -t/3.0f},
            },
            {   //7
                {-2/3.0f,        1/3.0f+2*t/3.0f, -t/3.0f},
                {-1/3.0f,        2/3.0f+t/3.0f,  -2*t/3.0f},
                {-t/3.0f,        2/3.0f,      -1/3.0f-2*t/3.0f},
                {-2*t/3.0f,       1/3.0f,      -2/3.0f-t/3.0f},
                {-1/3.0f-2*t/3.0f,   t/3.0f,      -2/3.0f},
                {-2/3.0f-t/3.0f,    2*t/3.0f,     -1/3.0f}
            },
            {   //8
                {-2/3.0f,        1/3.0f+2*t/3.0f, t/3.0f},
                {-1/3.0f,        2/3.0f+t/3.0f,  2*t/3.0f},
                {1/3.0f,         2/3.0f+t/3.0f,  2*t/3.0f},
                {2/3.0f,         1/3.0f+2*t/3.0f, t/3.0f},
                {1/3.0f,         t,        0.0f},
                {-1/3.0f,        t,        0.0f}
            },
            {   //9
                {-2/3.0f-t/3.0f,    2*t/3.0f,     1/3.0f},
                {-1/3.0f-2*t/3.0f,   t/3.0f,      2/3.0f},
                {-2*t/3.0f,       1/3.0f,      2/3.0f+t/3.0f},
                {-t/3.0f,        2/3.0f,      1/3.0f+2*t/3.0f},
                {-1/3.0f,        2/3.0f+t/3.0f,  2*t/3.0f},
                {-2/3.0f,        1/3.0f+2*t/3.0f, t/3.0f},
            },
            {   //10
                {-2/3.0f-t/3.0f,    2*t/3.0f,     -1/3.0f},
                {-1/3.0f-2*t/3.0f,   t/3.0f,      -2/3.0f},
                {-t,          0.0f,        -1/3.0f},
                {-t,          0.0f,        1/3.0f},
                {-1/3.0f-2*t/3.0f,   t/3.0f,      2/3.0f},
                {-2/3.0f-t/3.0f,    2*t/3.0f,     1/3.0f}
            },
            {   //11
                {-t/3.0f,        -2/3.0f,     -1/3.0f-2*t/3.0f},
                {-2*t/3.0f,      -1/3.0f,      -2/3.0f-t/3.0f,},
                {-2*t/3.0f,       1/3.0f,      -2/3.0f-t/3.0f},
                {-t/3.0f,         2/3.0f,      -1/3.0f-2*t/3.0f},
                {0.0f,           1/3.0f,      -t},
                {0.0f,           -1/3.0f,     -t}
            },
            {   //12
                {0.0f,           -1/3.0f,     -t},
                {0.0f,           1/3.0f,      -t},
                {t/3.0f,         2/3.0f,      -1/3.0f-2*t/3.0f},
                {2*t/3.0f,        1/3.0f,      -2/3.0f-t/3.0f},
                {2*t/3.0f,        -1/3.0f,     -2/3.0f-t/3.0f},
                {t/3.0f,         -2/3.0f,     -1/3.0f-2*t/3.0f}
            },
            {   //13
                {t/3.0f,         -2/3.0f,     -1/3.0f-2*t/3.0f},
                {2*t/3.0f,        -1/3.0f,     -2/3.0f-t/3.0f},
                {1/3.0f+2*t/3.0f,    -t/3.0f,      -2/3.0f},
                {2/3.0f+t/3.0f,     -2*t/3.0f,     -1/3.0f},
                {2/3.0f,         -1/3.0f-2*t/3.0f, -t/3.0f},
                {1/3.0f,         -2/3.0f-t/3.0f, -2*t/3.0f}
            },
            {   //14
                {0.0f,           -1/3.0f,       t},
                {0.0f,           1/3.0f,        t},
                {-t/3.0f,        2/3.0f,        1/3.0f+2*t/3.0f},
                {-2*t/3.0f,       1/3.0f,        2/3.0f+t/3.0f},
                {-2*t/3.0f,       -1/3.0f,        2/3.0f+t/3.0f},
                {-t/3.0f,        -2/3.0f,       1/3.0f+2*t/3.0f}
            },
            {   //15
                {t/3.0f,         -2/3.0f,       1/3.0f+2*t/3.0f},
                {2*t/3.0f,        -1/3.0f,       2/3.0f+t/3.0f},
                {2*t/3.0f,        1/3.0f,        2/3.0f+t/3.0f},
                {t/3.0f,         2/3.0f,        1/3.0f+2*t/3.0f},
                 {0.0f,           1/3.0f,        t},
                {0.0f,           -1/3.0f,       t}
         },
            {   //16
                {1/3.0f,         -2/3.0f-t/3.0f,   2*t/3.0f},
                {2/3.0f,         -1/3.0f-2*t/3.0f,  t/3.0f},
                {2/3.0f+t/3.0f,     -2*t/3.0f,      1/3.0f},
                {1/3.0f+2*t/3.0f,    -t/3.0f,       2/3.0f},
                {2*t/3.0f,        -1/3.0f,       2/3.0f+t/3.0f},
                {t/3.0f,         -2/3.0f,       1/3.0f+2*t/3.0f}
            },
            {   //17
                {1/3.0f,         2/3.0f+t/3.0f,    -2*t/3.0f},
                {2/3.0f,         1/3.0f+2*t/3.0f,   -t/3.0f},
                {2/3.0f+t/3.0f,     2*t/3.0f,       -1/3.0f},
                {1/3.0f+2*t/3.0f,    t/3.0f,        -2/3.0f},
                {2*t/3.0f,        1/3.0f,        -2/3.0f-t/3.0f},
                {t/3.0f,         2/3.0f,        -1/3.0f-2*t/3.0f}
            },
            {   //18
                {t/3.0f,         2/3.0f,        1/3.0f+2*t/3.0f},
                {2*t/3.0f,        1/3.0f,        2/3.0f+t/3.0f},
                {1/3.0f+2*t/3.0f,    t/3.0f,        2/3.0f},
                {2/3.0f+t/3.0f,     2*t/3.0f,       1/3.0f},
                {2/3.0f,         1/3.0f+2*t/3.0f,   t/3.0f},
                {1/3.0f,         2/3.0f+t/3.0f,    2*t/3.0f}
            },
            {   //19
                {2/3.0f+t/3.0f,     -2*t/3.0f,      -1/3.0f},
                {1/3.0f+2*t/3.0f,    -t/3.0f,       -2/3.0f},
                {t,           0.0f,          -1/3.0f},
                {t,           0.0f,          1/3.0f},
                {1/3.0f+2*t/3.0f,    -t/3.0f,       2/3.0f},
                {2/3.0f+t/3.0f,     -2*t/3.0f,      1/3.0f}
            },
            {   //20
                {2/3.0f+t/3.0f,     2*t/3.0f,       1/3.0f},
                {1/3.0f+2*t/3.0f,    t/3.0f,        2/3.0f},
                {t,           0.0f,          1/3.0f},
                {t,           0.0f,          -1/3.0f},
                {1/3.0f+2*t/3.0f,    t/3.0f,        -2/3.0f},
                {2/3.0f+t/3.0f,     2*t/3.0f,       -1/3.0f}
            }
        };

        static float[, ,] pol5 = new float[12, 5, 3] { 
            {   //21
                {-2/3.0f,        -1/3.0f-2*t/3.0f,  -t/3.0f},
                {-1/3.0f,        -t,         0.0f},
                {-2/3.0f,        -1/3.0f-2*t/3.0f,  t/3.0f},
                {-2/3.0f-t/3.0f,    -2*t/3.0f,      1/3.0f},
                {-2/3.0f-t/3.0f,    -2*t/3.0f,      -1/3.0f}
            },
            {   //22
                {-2/3.0f-t/3.0f,    2*t/3.0f,       -1/3.0f},
                {-2/3.0f-t/3.0f,    2*t/3.0f,       1/3.0f},
                {-2/3.0f,        1/3.0f+2*t/3.0f,   t/3.0f},
                {-1/3.0f,        t,          0.0f},
                {-2/3.0f,        1/3.0f+2*t/3.0f,   -t/3.0f}
            },
            {   //23
                {-t/3.0f,        -2/3.0f,       -1/3.0f-2*t/3.0f},
                {0.0f,           -1/3.0f,       -t},
                {t/3.0f,         -2/3.0f,       -1/3.0f-2*t/3.0f},
                {1/3.0f,         -2/3.0f-t/3.0f,   -2*t/3.0f},
                {-1/3.0f,        -2/3.0f-t/3.0f,   -2*t/3.0f}
            },
            {   //24
                {-1 / 3.0f, -2 / 3.0f - t / 3.0f, 2 * t / 3.0f},
                {1 / 3.0f, -2 / 3.0f - t / 3.0f, 2 * t / 3.0f},
                {t / 3.0f, -2 / 3.0f, 1 / 3.0f + 2 * t / 3.0f},
                {0.0f, -1 / 3.0f, t},
                {-t / 3.0f, -2 / 3.0f, 1 / 3.0f + 2 * t / 3.0f}
            },
            {   //25
                {-1/3.0f,        2/3.0f+t/3.0f,    -2*t/3.0f},
                {1/3.0f,         2/3.0f+t/3.0f,    -2*t/3.0f},
                {t/3.0f,         2/3.0f,        -1/3.0f-2*t/3.0f},
                {0.0f,           1/3.0f,        -t},
                {-t/3.0f,        2/3.0f,        -1/3.0f-2*t/3.0f}
            },
            {   //26
                {-t/3.0f,        2/3.0f,        1/3.0f+2*t/3.0f},
                {0.0f,           1/3.0f,        t},
                {t/3.0f,         2/3.0f,        1/3.0f+2*t/3.0f},
                {1/3.0f,         2/3.0f+t/3.0f,    2*t/3.0f},
                {-1/3.0f,        2/3.0f+t/3.0f,    2*t/3.0f}
            },
            {   //27
                {1/3.0f,         -t,         0.0f},
                {2/3.0f,         -1/3.0f-2*t/3.0f,  -t/3.0f},
                {2/3.0f+t/3.0f,     -2*t/3.0f,      -1/3.0f},
                {2/3.0f+t/3.0f,     -2*t/3.0f,      1/3.0f},
                {2/3.0f,         -1/3.0f-2*t/3.0f,  t/3.0f}
            },
            {   //28
                {2/3.0f,         1/3.0f+2*t/3.0f,   t/3.0f},
                {2/3.0f+t/3.0f,     2*t/3.0f,       1/3.0f},
                {2/3.0f+t/3.0f,     2*t/3.0f,       -1/3.0f},
                {2/3.0f,         1/3.0f+2*t/3.0f,   -t/3.0f},
                {1/3.0f,         t,          0.0f}
            },
            {   //29
                {-t,           0.0f,          -1/3.0f},
                {-1/3.0f-2*t/3.0f,   t/3.0f,        -2/3.0f},
                {-2*t/3.0f,       1/3.0f,        -2/3.0f-t/3.0f},
                {-2*t/3.0f,       -1/3.0f,       -2/3.0f-t/3.0f},
                {-1/3.0f-2*t/3.0f,   -t/3.0f,       -2/3.0f}
            },
            {   //30
                {-1/3.0f-2*t/3.0f,   -t/3.0f,       2/3.0f},
                {-2*t/3.0f,       -1/3.0f,       2/3.0f+t/3.0f},
                {-2*t/3.0f,       1/3.0f,        2/3.0f+t/3.0f},
                {-1/3.0f-2*t/3.0f,   t/3.0f,        2/3.0f},
                {-t,          0.0f,          1/3.0f}
            },
            {   //31
                {2*t/3.0f,        -1/3.0f,       -2/3.0f-t/3.0f},
                {2*t/3.0f,        1/3.0f,        -2/3.0f-t/3.0f},
                {1/3.0f+2*t/3.0f,    t/3.0f,        -2/3.0f},
                {t,           0.0f,          -1/3.0f},
                {1/3.0f+2*t/3.0f,    -t/3.0f,       -2/3.0f}
            },
            {   //32
                {1/3.0f+2*t/3.0f,    -t/3.0f,       2/3.0f},
                {t,           0.0f,          1/3.0f},
                {1/3.0f+2*t/3.0f,    t/3.0f,        2/3.0f},
                {2*t/3.0f,        1/3.0f,        2/3.0f+t/3.0f},
                {2*t/3.0f,        -1/3.0f,       2/3.0f+t/3.0f}
           }
        };
#endregion data
        static void Main(string[] args)
        {
            Glut.glutInit();
            // set double buffered
            Glut.glutInitDisplayMode(Glut.GLUT_RGB | Glut.GLUT_DOUBLE);
            Glut.glutInitWindowSize(320, 320);
            Glut.glutInitWindowPosition(50, 50);
            Glut.glutCreateWindow("Test");
            // once in a lifetime
            Init();
            PrepareData();
            LoadTexture();
            // define callbacks
            Glut.glutDisplayFunc(new Glut.DisplayCallback(Display));
            Glut.glutReshapeFunc(new Glut.ReshapeCallback(Reshape));
            Glut.glutMouseFunc(new Glut.MouseCallback(MouseState));
            Glut.glutMotionFunc(new Glut.MotionCallback(Dragging));
            // start the glut main loop
            Glut.glutMainLoop();
        }
#region init
        private static void Init(){
 
            // set up coloring model
            Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK,Gl.GL_FILL);
            Gl.glLightModeli(Gl.GL_LIGHT_MODEL_TWO_SIDE,1);
            
            // set up light
            float[] ambient = { 0.5f, 0.5f, 0.5f, 1.0f };
            float[] diffuse = { 1.0f, 1.0f, 1.0f, 1.0f };
            float[] position = { 200.0f, 300.0f, 100.0f, 0.0f };
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_AMBIENT, ambient);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, diffuse);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, position);
            
            // enable light 
            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_LIGHT0);

            // depth sorting
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            Gl.glDepthFunc(Gl.GL_LESS);

            // enable texture
            Gl.glEnable(Gl.GL_TEXTURE_2D);

            // auto normals
            //Gl.glEnable(Gl.GL_NORMALIZE);

            // background
            Gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
        }
        static void PrepareData()
        {
            // calculate normals for pol6:normal6 and pol5:normal5
            // Cross product is used:c=axb=(a2b3-a3b2, a3b1-a1b3, a1b2-a2b1)
            // The normals are normalized (length=1)
            for (int ix = 0; ix < 20; ix++)
            {
                float ax = pol6[ix, 2, 0] - pol6[ix, 0, 0];
                float ay = pol6[ix, 2, 1] - pol6[ix, 0, 1];
                float az = pol6[ix, 2, 2] - pol6[ix, 0, 2];
                float bx = pol6[ix, 3, 0] - pol6[ix, 0, 0];
                float by = pol6[ix, 3, 1] - pol6[ix, 0, 1];
                float bz = pol6[ix, 3, 2] - pol6[ix, 0, 2];
                float x = ay * bz - az * by;
                float y = az * bx - ax * bz;
                float z = ax * by - ay * bx;
                float l=(float)Math.Sqrt(x*x+y*y+z*z);
                normal6[ix,0]=x/l;
                normal6[ix,1]=y/l;
                normal6[ix,2]=z/l;
            }
            for (int ix = 0; ix < 12; ix++)
            {
                float ax = pol5[ix, 2, 0] - pol5[ix, 0, 0];
                float ay = pol5[ix, 2, 1] - pol5[ix, 0, 1];
                float az = pol5[ix, 2, 2] - pol5[ix, 0, 2];
                float bx = pol5[ix, 3, 0] - pol5[ix, 0, 0];
                float by = pol5[ix, 3, 1] - pol5[ix, 0, 1];
                float bz = pol5[ix, 3, 2] - pol5[ix, 0, 2];
                float x = ay * bz - az * by;
                float y = az * bx - ax * bz;
                float z = ax * by - ay * bx;
                float l=(float)Math.Sqrt(x*x+y*y+z*z);
                normal5[ix, 0] = x/l;
                normal5[ix, 1] = y/l;
                normal5[ix, 2] = z/l;
            }
            // calculate mapping coordinates for texture
            float d=1.0f;
            float cos60=(float)Math.Cos(Math.PI/3.0);//60 degrees
            float sin60=(float)Math.Sin(Math.PI/3.0);
            tex6[0,0]=d/3.0f;                   tex6[0,1]=0.0f;
            tex6[1,0]=2.0f*d/3.0f;              tex6[1,1]=0.0f;
            tex6[2,0]=d-(d/3.0f)*cos60;         tex6[2,1]=(d/3.0f)*sin60;
            tex6[3,0]=d-(2.0f*d/3.0f)*cos60;    tex6[3,1]=(2.0f*d/3.0f)*sin60;
            tex6[4,0]=(2.0f*d/3.0f)*cos60;      tex6[4,1]=(2.0f*d/3.0f)*sin60;
            tex6[5,0]=(d/3.0f)*cos60;           tex6[5,1]=(d/3.0f)*sin60;
        }
#endregion init
#region initbm
        static void LoadTexture()
        {
            // get a bitmap
            //string filename = Application.StartupPath + "\\Marrakech.jpg";
            //string filename = Application.StartupPath + "\\chesspiece.gif";
            string filename = Application.StartupPath + "\\ronaldinho.jpg";
            //string filename = Application.StartupPath + "\\cloth.bmp";
            Bitmap bm = new Bitmap(filename);
            
            // set up texture
            if (bm != null)
            {
                // get a place to store texture
                Gl.glGenTextures(1, out texture[0]); 

                // rotate the nbitmap
                bm.RotateFlip(RotateFlipType.RotateNoneFlipY);
                
                // Lock bitmap in rectangle
                Rectangle rectangle = new Rectangle(0, 0, bm.Width, bm.Height);
                // Fill the rectangle with a given format
                BitmapData bitmapData = bm.LockBits(rectangle, 
                    ImageLockMode.ReadOnly, 
                    PixelFormat.Format24bppRgb);

                // Tekstur generering
                Gl.glBindTexture(Gl.GL_TEXTURE_2D, texture[0]);
                
                Gl.glTexImage2D(Gl.GL_TEXTURE_2D, 0, Gl.GL_RGB8, 
                                bm.Width, bm.Height, 0, 
                                Gl.GL_BGR, Gl.GL_UNSIGNED_BYTE, bitmapData.Scan0);
                               
                /* Gl.glTexImage2D(Gl.GL_TEXTURE_2D, 0, Gl.GL_RGB8,
                                bm.Width, bm.Height, 0,
                                Gl.GL_RGB, Gl.GL_UNSIGNED_BYTE, bitmapData.Scan0);
                */
                Gl.glTexParameteri(Gl.GL_TEXTURE_2D, 
                                Gl.GL_TEXTURE_MIN_FILTER, Gl.GL_LINEAR);
                Gl.glTexParameteri(Gl.GL_TEXTURE_2D, 
                                Gl.GL_TEXTURE_MAG_FILTER, Gl.GL_LINEAR);
                //Gl.glTexEnvi(Gl.GL_TEXTURE_ENV, Gl.GL_TEXTURE_ENV_MODE, Gl.GL_DECAL);
                //Gl.glTexEnvi(Gl.GL_TEXTURE_ENV, Gl.GL_TEXTURE_ENV_MODE, Gl.GL_MODULATE);

                if (bm != null)
                {
                    bm.UnlockBits(bitmapData); // Release
                    bm.Dispose();              // Dispose 
                }
            }
            else
                MessageBox.Show("Ingen bitmap", "ERROR",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
#endregion initbm

        #region materials
        private static void SetWhiteMaterial()
        {
            float[] amb={0.58f,0.59f,0.59f,1.0f};
            float[] diff={0.71f,0.72f,0.71f,1.0f};
            float[] spec ={ 0.508273f, 0.508273f, 0.508273f, 1.0f };
            float shine=51.2f;           
            //float[] amb={0.31f,0.32f,0.32f,1.0f};
            //float[] diff={0.59f,0.59f,0.0f,1.0f};
            //float[] spec={0.11f,0.11f,0.11f,1.0f};
            //float shine=128f;
            Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, amb);
            Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, diff);
            Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, spec);
            Gl.glMaterialf(Gl.GL_FRONT, Gl.GL_SHININESS, shine);

            /* backside*/
            float[] backamb={0.0215f,0.1745f,0.0215f,1.0f};
            float[] backdiff={0.07568f,0.61424f,0.07568f,1.0f};
            float[] backspec={0.633f,0.727811f,0.633f,1.0f};
            float backshine=76.8f;
            Gl.glMaterialfv(Gl.GL_BACK,Gl.GL_AMBIENT,backamb);
            Gl.glMaterialfv(Gl.GL_BACK,Gl.GL_DIFFUSE,backdiff);
            Gl.glMaterialfv(Gl.GL_BACK,Gl.GL_SPECULAR,backspec);
            Gl.glMaterialf(Gl.GL_BACK,Gl.GL_SHININESS,backshine);        
        }
        private static void SetBlackMaterial()
        {
            float[] amb={0.11f,0.1f,0.31f,1.0f};
            float[] diff={0.38f,0.38f,0.8f,1.0f};
            float[] spec={0.19f,0.14f,0.49f,1.0f};
            float shine=79f;
            Gl.glMaterialfv(Gl.GL_FRONT,Gl.GL_AMBIENT,amb);
            Gl.glMaterialfv(Gl.GL_FRONT,Gl.GL_DIFFUSE,diff);
            Gl.glMaterialfv(Gl.GL_FRONT,Gl.GL_SPECULAR,spec);
            Gl.glMaterialf(Gl.GL_FRONT,Gl.GL_SHININESS,shine);
            /* backside*/
            float[] backamb ={ 0.0215f, 0.1745f, 0.0215f, 1.0f };
            float[] backdiff ={ 0.07568f, 0.61424f, 0.07568f, 1.0f };
            float[] backspec ={ 0.633f, 0.727811f, 0.633f, 1.0f };
            float backshine = 76.8f;
            Gl.glMaterialfv(Gl.GL_BACK, Gl.GL_AMBIENT, backamb);
            Gl.glMaterialfv(Gl.GL_BACK, Gl.GL_DIFFUSE, backdiff);
            Gl.glMaterialfv(Gl.GL_BACK, Gl.GL_SPECULAR, backspec);
            Gl.glMaterialf(Gl.GL_BACK, Gl.GL_SHININESS, backshine);
        }
#endregion materials

#region callbacks
        private static void Display()
        {
            Gl.glClear( Gl.GL_COLOR_BUFFER_BIT | 
                        Gl.GL_DEPTH_BUFFER_BIT | 
                        Gl.GL_ACCUM_BUFFER_BIT);
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glLoadIdentity();
            Glu.gluLookAt(0, 0, 7, // eyepos
                          0, 0, 0, // look at
                          0, 1, 0);// up-vector
            Gl.glRotatef(v1, 1.0f, 0.0f, 0.0f);
            Gl.glRotatef(v2, 0.0f, 0.0f, 1.0f);
            DrawBuckyBall();
            Glut.glutSwapBuffers();
        }

        private static void Reshape(int width, int height)
        {
            Gl.glMatrixMode(Gl.GL_PROJECTION);            
            Gl.glLoadIdentity();
            Gl.glViewport(0, 0, width, height);
            Glu.gluPerspective(30.0f, (double)width/height, 0.1f, 70.0f);
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
        }
        
        // when mousestate changes
        private static void MouseState(int button, int state, int x, int y)
        {
            // we care only for left button
            if (button != Glut.GLUT_LEFT_BUTTON)
                return;
            MouseIsDown=!MouseIsDown;
            LastX = x;
            LastY = y;
        }
        
        // when mouse is moved
        private static void Dragging(int x, int y)
        {
            if (MouseIsDown)
            {
                if (Math.Abs(LastX - x) > Math.Abs(LastY - y))
                    v2 += 3.0f;
                else
                    v1 += 3.0f;
                LastX = x;
                LastY = y;
                Glut.glutPostRedisplay();
            }
        }
#endregion callbacks
#region draw
        // 2 versions:
        // First, unmasked, version draw the buckyball with texture
        // Second, masked, version draw the buckyball as a grid with spheres in cornes
        // Change masking to experiment

        static void DrawBuckyBall()
        {
            int mode = Gl.GL_POLYGON;
            // polygons with 6 edges
            SetWhiteMaterial();
            for (int p = 0; p < 20; p++)
            {
                Gl.glBegin(mode);
                Gl.glNormal3f(normal6[p, 0], normal6[p, 1], normal6[p, 2]);
                for (int ix = 0; ix < 6; ix++)
                {
                    Gl.glTexCoord2f(tex6[ix, 0], tex6[ix, 1]);
                    Gl.glVertex3f(pol6[p, ix, 0], pol6[p, ix, 1], pol6[p, ix, 2]);
                }
                Gl.glEnd();
            }

            // polygons with 5 edges, no texture
            SetBlackMaterial();
            for (int p = 0; p < 12; p++)
            {
                Gl.glBegin(mode);
                Gl.glNormal3f(normal5[p, 0], normal5[p, 1], normal5[p, 2]);
                for (int ix = 0; ix < 5; ix++)
                    Gl.glVertex3f(pol5[p, ix, 0], pol5[p, ix, 1], pol5[p, ix, 2]);
                Gl.glEnd();
            }
        }

        //static void DrawBuckyBall()
        //{
        //    Gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
        //    int mode = Gl.GL_LINE_LOOP;
        //    // polygons with 6 edges
        //    SetWhiteMaterial();
        //    for (int p = 0; p < 20; p++)
        //    {
        //        Gl.glBegin(mode);
        //        for (int ix = 0; ix < 6; ix++)
        //        {
        //            Gl.glVertex3f(pol6[p, ix, 0], pol6[p, ix, 1], pol6[p, ix, 2]);
        //        }
        //        Gl.glEnd();
        //    }

        //    // polygons with 5 edges
        //    for (int p = 0; p < 12; p++)
        //    {
        //        Gl.glBegin(mode);
        //        for (int ix = 0; ix < 5; ix++)
        //            Gl.glVertex3f(pol5[p, ix, 0], pol5[p, ix, 1], pol5[p, ix, 2]);
        //        Gl.glEnd();
        //    }
        //    // spheres in corners
        //    SetBlackMaterial();
        //    Glu.GLUquadric q;
        //    q = Glu.gluNewQuadric(); // note this
        //    for (int p = 0; p < 12; p++)
        //        for (int ix = 0; ix < 5; ix++)
        //        {
        //            Gl.glPushMatrix();
        //            Gl.glTranslatef(pol5[p, ix, 0], pol5[p, ix, 1], pol5[p, ix, 2]);
        //            Glu.gluSphere(q, 0.1, 20, 20);
        //            Gl.glPopMatrix();
        //        }
        //    Glu.gluDeleteQuadric(q);

        //}
#endregion draw     
    }

 }
